package cn.com.qmth.scancloud.service.impl;

import cn.com.qmth.scancloud.bean.ScannerLoginInfo;
import cn.com.qmth.scancloud.bean.User;
import cn.com.qmth.scancloud.client.MarkingcloudApiClient;
import cn.com.qmth.scancloud.client.MarkingcloudProperties;
import cn.com.qmth.scancloud.entity.SystemConfigEntity;
import cn.com.qmth.scancloud.entity.UserEntity;
import cn.com.qmth.scancloud.enums.Role;
import cn.com.qmth.scancloud.enums.SystemMode;
import cn.com.qmth.scancloud.exception.ForbiddenExceptions;
import cn.com.qmth.scancloud.exception.UnauthorizedExceptions;
import cn.com.qmth.scancloud.model.UserInfo;
import cn.com.qmth.scancloud.service.AuthService;
import cn.com.qmth.scancloud.service.ScannerService;
import cn.com.qmth.scancloud.service.SessionService;
import cn.com.qmth.scancloud.service.UserService;
import com.qmth.boot.core.exception.ParameterException;
import com.qmth.boot.core.exception.StatusException;
import com.qmth.boot.core.security.annotation.AuthorizationComponent;
import com.qmth.boot.core.security.service.AuthorizationService;
import com.qmth.boot.tools.signature.SignatureType;
import org.apache.commons.lang3.RandomStringUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

@Service
@AuthorizationComponent
public class AuthServiceImpl implements AuthorizationService<User>, AuthService {

    protected static final Logger log = LoggerFactory.getLogger(AuthServiceImpl.class);

    @Autowired
    private UserService userService;

    @Autowired
    private SessionService sessionService;

    @Autowired
    private ScannerService scannerService;

    @Autowired
    private SystemConfigServiceImpl systemConfigService;

    @Autowired
    private MarkingcloudApiClient apiClient;

    @Autowired
    private MarkingcloudProperties markingcloudProperties;

    @Override
    public User findByIdentity(String identity, SignatureType type, String path) {
        User user = sessionService.getSessionUser(identity);
        return user;
    }

    @Override
    public boolean hasPermission(User user, String path) {
        sessionService.updateUserSession(user);
        return true;
    }

    @Transactional
    @Override
    public User login(String ip, ScannerLoginInfo loginInfo) {
        if (StringUtils.isBlank(ip)) {
            throw new ParameterException("IP获取失败");
        }
        SystemConfigEntity config = systemConfigService.findConfig();
        if (config != null) {
            if (config.getScannerEnableLogin() != null && !config.getScannerEnableLogin()) {
                throw ForbiddenExceptions.SCANNER_LOGIN_DISABLE;
            }
            if (StringUtils.isNotEmpty(config.getScannerPassword())) {
                if (loginInfo.getPassword() == null) {
                    throw UnauthorizedExceptions.SCANNER_PASSWORD_INVALID;
                }
                if (!config.getScannerPassword().equals(loginInfo.getPassword())) {
                    throw UnauthorizedExceptions.SCANNER_PASSWORD_INVALID;
                }
            }
        }
        User user = new User();
        user.setRole(Role.SCANNER);
        user.setAccount(ip);
        user.setAccessToken(RandomStringUtils.randomAlphanumeric(32));
        user.buildKey();
        scannerService.onLogin(ip);
        sessionService.userLogin(user);
        return user;
    }

    @Override
    public User adminLogin(String loginName, String password, String device) {
        SystemConfigEntity conf = systemConfigService.findConfig();
        if (SystemMode.STANDALONE.equals(conf.getSystemMode())) {
            UserEntity userEntity = userService.getByLoginName(loginName);
            if (userEntity == null) {
                throw new ParameterException("用户不存在");
            }
            if (!userEntity.getEnable()) {
                throw new ParameterException("用户已禁用");
            }
            if (!password.equals(userEntity.getPassword())) {
                throw new ParameterException("密码错误");
            }
            userEntity.setDevice(device);
            userService.saveOrUpdate(userEntity);
            User user = new User();
            user.setRole(userEntity.getRole());
            user.setAccount(loginName);
            user.setAccessToken(RandomStringUtils.randomAlphanumeric(32));
            user.buildKey();
            user.setId(userEntity.getId());
            user.setName(userEntity.getName());
            user.setSchoolId(userEntity.getSchoolId());
            sessionService.userLogin(user);
            return user;
        } else if (SystemMode.MARKING_CLOUD.equals(conf.getSystemMode())) {
            User user = new User();
            user.setAccount(loginName);
            user.setAccessToken(RandomStringUtils.randomAlphanumeric(32));
            UserEntity userEntity = userService.getByLoginName(loginName);
            if (userEntity != null) {
                if (!userEntity.getEnable()) {
                    throw new ParameterException("用户已禁用");
                }
                if (!password.equals(userEntity.getPassword())) {
                    throw new ParameterException("密码错误");
                }
                user.setRole(userEntity.getRole());
                user.buildKey();
                user.setId(userEntity.getId());
                user.setName(userEntity.getName());
                user.setSchoolId(userEntity.getSchoolId());
                userEntity.setDevice(device);
                userService.saveOrUpdate(userEntity);
            } else {
                try {
                    UserInfo userInfo = apiClient.login(loginName, password);
                    user.setRole(userInfo.getRole());
                    user.buildKey();
                    //为了避免与本地用户ID重复，云阅卷用户ID统一加10000
                    user.setId(10000 + userInfo.getId());
                    user.setName(userInfo.getName());
                    user.setSchoolId(userInfo.getSchoolId());
                    user.setMarkingCloudToken(userInfo.getToken());
                    //登录后更新fileServer属性
                    markingcloudProperties.setFileServer(userInfo.getFileServer());
                } catch (Exception e) {
                    log.error("markingcloud user login error", e);
                    throw UnauthorizedExceptions.MARKINGCLOUD_LOGIN_ERROR;
                }
            }
            sessionService.userLogin(user);
            return user;
        } else {
            throw new StatusException("系统配置模式错误");
        }
    }

}
